/*
 *  OpenVPN-GUI -- A Windows GUI for OpenVPN.
 *
 *  Copyright (C) 2004 Mathias Sundman <mathias@nilings.se>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program (see the file COPYING included with this
 *  distribution); if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <windows.h>
#include "main.h"
#include "openvpn-gui-res.h"
#include "options.h"

static bool
match (const WIN32_FIND_DATA *find, const char *ext)
{
  int i;

  if (find->dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
    return false;

  if (!strlen (ext))
    return true;

  i = strlen (find->cFileName) - strlen (ext) - 1;
  if (i < 1)
    return false;

  return find->cFileName[i] == '.' && !strcasecmp (find->cFileName + i + 1, ext);
}

/*
 * Modify the extension on a filename.
 */
static bool
modext (char *dest, int size, const char *src, const char *newext)
{
  int i;

  if (size > 0 && (strlen (src) + 1) <= size)
    {
      strcpy (dest, src);
      dest [size - 1] = '\0';
      i = strlen (dest);
      while (--i >= 0)
	{
	  if (dest[i] == '\\')
	    break;
	  if (dest[i] == '.')
	    {
	      dest[i] = '\0';
	      break;
	    }
	}
      if (strlen (dest) + strlen(newext) + 2 <= size)
	{
	  strcat (dest, ".");
	  strcat (dest, newext);
	  return true;
	}
      dest [0] = '\0';
    }
  return false;
}

int
BuildFileList()
{
  WIN32_FIND_DATA find_obj;
  HANDLE find_handle;
  BOOL more_files;
  char find_string[MAX_PATH];
  int i;
  char log_file[MAX_PATH];
  extern struct options o;


  /* Reset config counter */
  o.num_configs=0;
  
  mysnprintf (find_string, "%s\\*", o.config_dir);

  find_handle = FindFirstFile (find_string, &find_obj);
  if (find_handle == INVALID_HANDLE_VALUE)
    {
      return(true);
    }

  /*
   * Loop over each config file
   */
  do {
    if (o.num_configs >= MAX_CONFIGS) {
      /* too many configs */
      ShowLocalizedMsg(GUI_NAME, ERR_TO_MANY_CONFIGS, MAX_CONFIGS);
      break;
    }

    /* does file have the correct type and extension? */
    if (match (&find_obj, o.ext_string))
      {
        /* Save config file name */
        strncpy(o.cnn[o.num_configs].config_file, find_obj.cFileName, sizeof(o.cnn[o.num_configs].config_file));

        /* Save connection name (config_name - extension) */
        strncpy(o.cnn[o.num_configs].config_name, o.cnn[o.num_configs].config_file,
                sizeof(o.cnn[o.num_configs].config_name));
        o.cnn[o.num_configs].config_name[strlen(o.cnn[o.num_configs].config_name) - 
                                         (strlen(o.ext_string)+1)]=0;

        /* get log file pathname */
        if (!modext (log_file, sizeof (log_file), o.cnn[o.num_configs].config_file, "log"))
          {
	    /* cannot construct logfile-name */
            ShowLocalizedMsg (GUI_NAME, ERR_CANNOT_CONSTRUCT_LOG, o.cnn[o.num_configs].config_file);
            goto finish;
          }
        mysnprintf (o.cnn[o.num_configs].log_path, "%s\\%s", o.log_dir, log_file);

        /* Set askpass to true as default */
        o.cnn[o.num_configs].ask_pass = true;

        /* Check if connection should be autostarted */
        for (i=0; (i < MAX_CONFIGS) && o.auto_connect[i]; i++)
          {
            if (strcasecmp(o.cnn[o.num_configs].config_file, o.auto_connect[i]) == 0)
              {
                o.cnn[o.num_configs].auto_connect = true;
                break;
              }
          }

        o.num_configs++;
      }
      
    /* more files to process? */
    more_files = FindNextFile (find_handle, &find_obj);
  } while (more_files);
    
  FindClose (find_handle);
  
  return(true);

finish:
  return(false);
}

